*! 1.0.0
program define CorrSample
	version 10.0
	syntax anything [, mean(name) cov(name) corr(name) exch(real 0.0) DOUble]

	tokenize "`anything'"
	local n "`1'"
	local T "`2'"

	confirm integer number `n'
	confirm integer number `T'

	if "`3'" != "" {
		di as err "Only two integer arguments are allowed"
		exit 198
	}

	if `n' < 1 {
		di as err "First argument must be a nonnegative integer"
		exit 198
	}
	if `T' < 1 {
		di as err "Second argument must be a nonnegative integer"
		exit 198
	}

	if "`mean'" != "" {
		confirm matrix `mean'
		if rowsof(`mean') != 1 | colsof(`mean') != `T' {
			di as err "mean() argument must name a 1x`T' matrix"
			exit 503
		}
		matrix B = `mean'
	}
	else {
		matrix B = J(1,`T',0)
	}

	if "`corr'" != "" {
		if "`cov'" != "" {
			di as err "Only one of corr() and cov() is allowed"
			exit 198
		}
		if rowsof(`corr') != `T' | colsof(`corr') != `T' {
			di as err "corr() argument must name a square matrix"
			exit 503
		}
		matrix S = `corr'
	}

	else {
		if "`cov'" != "" {
			if rowsof(`cov') != `T' | colsof(`cov') != `T' {
				di as err "cov() argument must name a square matrix"
				exit 503
			}
			matrix S = `cov'
		}
		else {
			matrix S = (1-`exch')*I(`T') + `exch'*J(`T',`T',1)
			matrix S = _pi*_pi/3 * S
		}
	}

	quietly {
		drop _all
		set obs `n'

		tempvar unif
		gen double `unif' = .
		local i 1
		while `i' <= `T' {
			replace `unif' = uniform()
			gen `double' err`i' =  invnorm(uniform()) // log(`unif'/(1-`unif')) // invnorm(uniform())
			gen `double' z`i' = 0
			local i=`i'+1
		}

		corr err*, cov
		matrix V  = r(C)
		matrix D  = cholesky(diag(vecdiag(S)))
		matrix R  = syminv(D)*S*syminv(D)
		matrix C1 = cholesky(invsym(V))
		matrix C2 = cholesky(R)
		matrix CC = C1*C2'*D

		local j 1
		while `j' <= `T' {
			summ err`j'
			replace err`j' = err`j'-r(mean)
			local j = `j'+1
		}

		local j 1
		while `j' <= `T' {
			local exp "0"
			local i 1
			while `i' <= `T' {
				local exp "`exp'+err`i'*C1[`i',`j']"
				local i = `i'+1
			}
			replace z`j' = `exp'
			local j = `j'+1
		}

		local j 1
		while `j' <= `T' {
			local exp "0"
			local i 1
			while `i' <= `T' {
				local exp "`exp'+z`i'*C2[`j',`i']"
				local i = `i'+1
			}
			replace err`j' = D[`j',`j']*(`exp') + B[1,`j']
			label var err`j' "Component for time=`j'"
			local j = `j'+1
		}

		drop z*
		gen panel = _n
		reshape i   panel
		reshape j   time 
		reshape xij err
		reshape long
		xtset panel time
		reshape wide
	}
end
