
* Alternate Count Data Model Chapter
drop _all

set obs 21
scalar mu     = 4
scalar alpha1 = 0.5
scalar alpha2 = 1.5
scalar p1     = 1/(1+alpha1*mu)
scalar p2     = 1/(1+alpha2*mu)
generate y = _n-1
generate double fy0p0 = exp(-mu)*mu^y/exp(lngamma(y+1))
generate double fy0p5 = exp(lngamma(y+1/alpha1)-lngamma(y+1)-lngamma(1/alpha1))*p1^(1/alpha1)*(1-p1)^y
generate double fy1p5 = exp(lngamma(y+1/alpha2)-lngamma(y+1)-lngamma(1/alpha2))*p2^(1/alpha2)*(1-p2)^y
label var fy0p0 "NB-alpha=0.0"
label var fy0p5 "NB-alpha=0.5"
label var fy1p5 "NB-alpha=1.5"
twoway (scatter fy0p0 y, c(l)) (scatter fy0p5 y, c(l)) (scatter fy1p5 y, c(l)), title("Mean=4: Count Data Models") 
graph export nbplot.eps, replace

drop _all
set seed 2349
set obs 50000
generate x1 = invnormal(runiform())
generate x2 = invnormal(runiform())
generate xb = -1.0 + 0.5*x1 + 1.5*x2
generate lambda = exp(xb)
generate yp = rpoisson(lambda)

poisson yp x1 x2

poisson yp x1 x2 if y>0, nolog

tpoisson yp x1 x2 if y>0, nolog

tabulate yp if yp <= 15

zip yp x1 x2, inflate(x1 x2) nolog


clear
set obs 50000
set seed 1234
generate x1 = rnormal()      /* x1 ~ N(0,1) */
generate x2 = rnormal()      /* x2 ~ N(0,1) */
generate yp = rpoisson(exp(2.0+0.75*x1-1.25*x2)) /* yp ~ Poisson           */
generate p  = 1/(1+exp((0.2+0.9*x1+0.1*x2)))     /* p=P(B=0) for Bernoulli */
generate yb = runiform() < p                     /* yb ~ Bernoulli(1-p)    */
generate yz = yb*yp                              /* yz ~ ZIP               */

zip yz x1 x2, inflate(x1 x2) nolog
save junk, replace

use rwm1984, clear
zigp docvis outwork female age married, inflate(outwork female age married) vuong nolog
abic

*10
quietly gpoisson docvis outwork female age married
abic

use junk, clear
capture program drop HurdPoissLogitLL
program HurdPoissLogitLL
    args lnf beta1 beta2
    quietly {
        tempvar pi mu
        generate double `mu' = exp(`beta1')
        generate double `pi' = exp(`beta2')
        replace `lnf' = cond($ML_y1==0,              ///
            ln(`pi'/(1+`pi')),                       ///
            ln(1/(1+`pi')) -`mu' + $ML_y1 * `beta1'  ///
            - lngamma($ML_y1 + 1) - ln(1 - exp(-`mu')) )
    }
end
generate ypgt0 = yp>0
ml model lf HurdPoissLogitLL (yp:yp=x1 x2) (logit:ypgt0=x1 x2)
ml maximize, search(on) nolog title(Poisson Logistic Hurdle Regression)

generate yp0 = yp==0
logit yz0 x1 x2, nolog

tpoisson yp x1 x2 if yp>0, nolog

ml model lf HurdPoissLogitLL (yp:yp=x1 x2) (logit:ypgt0=x1 x2)
ml maximize, search(on) nolog title(Poisson Logistic Hurdle Regression) irr
predict xb1, eq(#1)       /* zero-truncated Poisson prediction */
predict xb2, eq(#2)       /* logit prediction                  */
generate mu1 = exp(xb1)        /* predicted ZTP count          */
generate mu2 = 1/(1+exp(xb2))  /* predicted Pr(yp=0)           */

list yp mu1 yb mu2 in 1000/1020


hplogit yp x1 x2, nolog

clear
set obs 50000
set seed 1234
generate x1 = rnormal()       /* x1 ~ N(0,1)    */
generate x2 = rnormal()       /* x2 ~ N(0,1)    */
generate z1 = runiform() < .2 /* z1 ~ Ber(0.20) */
generate z2 = runiform() < .4 /* z2 ~ Ber(0.40) */
generate yp = rpoisson(exp(2.0+0.75*x1-1.25*x2)) /* yp ~ Poisson           */
drop if yp==0                                    /* yp ~ ZTP               */
generate p  = 1/(1+exp((-1.0+0.9*z1+0.1*z2)))    /* p=P(B=0) for Bernoulli */
generate yb = runiform() < p                     /* yb ~ Bernoulli(p)      */
generate yh = yp*yb                              /* yh ~ ZIP               */

generate yhgt0 =yh>0
ml model lf HurdPoissLogitLL (yh:yh=x1 x2) (logit:yhgt0=z1 z2)
ml maximize, search(on) nolog title(Poisson Logistic Hurdle Regression)


* Negbin(p)
clear
set seed 2942
set obs  50000
generate x1 = rnormal()
generate x2 = rnormal()
generate expxb1 = exp(1.00 - 1.00*x1 + 1.25*x2)
generate ap1    = 0.33*expxb1
generate expxb2 = exp(1.00 + 1.00*x1 + 1.00*x2)
generate ap2    = 0.25
generate ynb1 = rpoisson(rgamma(ap1, 1/ap1) * expxb1) // ynb1 ~ NB(1) 
generate ynb2 = rpoisson(rgamma(1/ap2, ap2) * expxb2) // ynb2 ~ NB(2)

capture program drop NegbinpLL
program NegbinpLL
    args lnf xb P lntheta
    quietly {
        tempvar g r lambda theta
        generate double `theta'  = exp(`lntheta')
        generate double `lambda' = exp(`xb') 
        generate double `g' = `theta'*`lambda'^(2-`P')
        generate double `r' = `g'/(`g'+`lambda')
        replace `lnf' = lngamma($ML_y1+`g') - lngamma(`g') ///
            - lngamma($ML_y1+1) + `g'*ln(`r') + $ML_y1*ln(1-`r')
    }
end



*20
quietly nbreg ynb1 x1 x2, disp(cons)  // Run NB(1) model
matrix b = e(b)                       // Get coefficient vector: beta,lntheta
local nc = colsof(b)
local ncp1 = `nc'+1                   // Start vals NB(P=1): beta,1,1/lntheta
local lntheta = 1/b[1,`nc']
matrix b[1,`nc'] = 1                  // Insert 1 for power
matrix b = b,`lntheta'                // Add lntheta to end of vector
ml model lf NegbinpLL (ynb1:ynb1 = x1 x2) /P /lntheta
ml init b, copy
ml maximize, search(off) nolog title(Negative Binomial(p) Regression) ///
    diparm(lntheta, exp label("theta"))

quietly nbreg ynb2 x1 x2     // Run NB(2) model
matrix b = e(b)              // Get coefficient vector: beta,lntheta
local nc = colsof(b)
local ncp1 = `nc'+1          // Start vals NB(P=2): beta,2,lntheta
local lntheta = b[1,`nc']
matrix b[1,`nc'] = 2         // Insert 2 for power
matrix b = b,`lntheta'       // Add lntheta to end of vector
ml model lf NegbinpLL (ynb2:ynb2 = x1 x2) /P /lntheta
ml init b, copy
ml maximize, search(off) nolog title(Negative Binomial(p) Regression) ///
    diparm(lntheta, exp label("theta"))


use rwm5panel, clear
quietly nbreg docvis outwork age female edlevel3 edlevel4, dispersion(constant)
estimates store A1
quietly nbreg docvis outwork age female edlevel3 edlevel4
estimates store A2
matrix b = e(b)
local nc = colsof(b)
local ncp1 = `nc'+1
local lntheta = 1/b[1,`nc']
matrix b[1,`nc'] = 2
matrix b = b,`lntheta'
ml model lf NegbinpLL (docvis:docvis = outwork age female edlevel3 edlevel4) /P /lntheta
ml init b, copy
ml maximize, search(off) nolog title(Negative Binomial(p) Regression) ///
    diparm(lntheta, exp label("theta")) irr
estimates store B

estimates restore A1
ml display, irr noheader
lrtest A1 B, force

estimates restore A2
ml display, irr noheader
lrtest A2 B, force

estimates restore B
test [P]_cons=1
test [P]_cons=2

use medpar, clear
nbreg los hmo white type2 type3 died, nolog

gnbreg los hmo white type2 type3 died, lnalpha(type2 type3 died) nolog


glm los hmo white type2 type3 died, family(nbinomial .4339959) notable nolog

glm los hmo white type2 type3 died, family(poisson) notable nolog

*30
gnbreg los hmo white type2 type3, lnalpha(died) irr nolog

gnbreg 

gpoisson los hmo white type2 type3, irr nolog

use doll, clear
glm deaths smokes a2-a5, lnoffset(pyears) family(poi) irls nolog 
predict double pearson, pearson
predict double yhatp

use doll, clear
gpoisson deaths smokes a2-a5, exposure(pyears) irr nolog 


use azdrg112, clear
gpoisson los male age75 emergency, nolog
abic

quietly poisson los male age75 emergency
abic

use azdrg112, clear
pigreg los male age75 emergency, nolog
abic

pigreg, irr

use medparc, clear
describe

tabulate cenx

list los cenx if cenx == -1

tabulate los if los > 60

tabulate los if los > 20 & los < 30

capture program drop CensPoisson
generate cens = cond(cenx==-1,-1,los)
program CensPoisson
    args lnf xb
    quietly {
        tempvar p1 p2
        generate double `p1' = min(1,gammap($ML_y1,exp(`xb')))
        generate double `p2' = cond($ML_y2==-1,0,gammap($ML_y2+1,exp(`xb')))
        replace `lnf' = ln(`p1'-`p2') if $ML_samp
    }
end
ml model lf CensPoisson (los:los cens=hmo white type2 type3)
ml maximize, search(on) nolog title(Censored Poisson Regression)

capture drop covpattern
egen covpattern = group(hmo white type2 type3)
table covpattern if e(sample), contents(freq mean los sd los)


capture program drop CensNegBin
program CensNegBin
    args todo b lnf 
    quietly {
        tempvar xb lnalpha r p ll
        mleval `xb'      = `b', eq(1)
        mleval `lnalpha' = `b', eq(2)
        generate double `r' = exp(-`lnalpha')
        generate double `p' = 1/(1+exp(-`xb'-`lnalpha'))
	   tempvar p1 p2
	   generate double `p1' = cond($ML_y1==0,1,ibeta($ML_y1,`r',`p'))
	   generate double `p2' = cond($ML_y2==-1,0,ibeta($ML_y2+1,`r',`p'))
        replace `lnf' =  ln(`p1'-`p2')
    }
end
ml model gf0 CensNegBin (los:los cens=hmo white type2 type3) /lnalpha
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr
abic

ml model gf0 CensNegBin (los:los los=hmo white type2 type3) /lnalpha
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr 
abic

ml model gf0 CensNegBin (los:los cens=hmo white type2 type3) /lnalpha, ///
    vce(cluster provnum)
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr
abic

replace cens  = -1 if los>=30  // Censored at 30
generate los30 = min(los,30)   // Dataset-censor value
ml model gf0 CensNegBin (los:los30 cens=hmo white type2 type3) /lnalpha, ///
    vce(cluster provnum)
ml maximize, search(on) nolog ti(Censored Negative Binomial Regression) ///
    diparm(lnalpha, exp label("alpha")) irr 

use azdrg112, clear
quietly fmm los male emergency age75, components(2) mixtureof(poisson)
fmm
abic

predict mean1, equation(component1)
predict mean2, equation(component2)
summarize los mean1 mean2

generate fmmhat = mean1*e(pi1_est) + mean2*e(pi2_est)
summarize fmmhat

clear
set obs 75000
set seed 2342
generate n1 = rnormal()
generate n2 = rnormal()
generate b1 = runiform() < .4
generate b2 = runiform() < .6
generate expxb1 = exp(2.00 + 1.50*n1 + -0.5*n2 + 3.00*b1 - 4.0*b2)
generate ap1    = 0.66
generate expxb2 = exp(1.00 + 1.00*n1 + 1.00*b1 - 2.0*b2)
generate ap2    = 0.25
generate expxb3 = exp(1.00 - 1.00*n1 + 1.25*n2)
generate ap3    = 0.33
generate ynb1 = rpoisson(rgamma(1/ap1, ap1) * expxb1)   // ynb1 ~ NB(2) 
generate ynb2 = rpoisson(rgamma(1/ap2, ap2) * expxb2)   // ynb2 ~ NB(2)
generate ynb3 = rpoisson(rgamma(1/ap3, ap3) * expxb3)   // ynb3 ~ NB(2)
generate mix = runiform()
generate yfmm = (mix>0.40)*ynb1 + (mix<0.25)*ynb2 + (mix>0.25 & mix<0.40)*ynb3 

quietly nbreg yfmm n1 n2 b1 b2
matrix b = e(b)
matrix b = b, b, b, -0.70, 0.70

local lna1 = ln(0.66)
local lna2 = ln(0.25)
local lna3 = ln(0.33)
matrix b = (1.5, -.5, 3, -4, 2, `lna1', 1, 0, 1, -2, 1, `lna2', -1, 1.25, 0, 0, 1, `lna3', -.7, .7)

quietly fmm yfmm n1 n2 b1 b2, components(3) mixtureof(negbin2) from(b)
fmm

