*! version 1.0.0
program define drawbinmodel
	syntax anything(name=args) [, PRobit CORR(string) ]

	quietly {
		local nargs : word count `args'

		if `nargs' != 2 {
			noi di as err "2 args needed"
			exit 198
		}

		local n : word 1 of `args'
		local t : word 2 of `args'
		
		if `n' < 3 | `t' < 2 | `t' > 8 {
			noi di as err "limits: n>=3 and 2<=t<=8"
			exit 198
		}
		
		if "`corr'" == "" | "`corr'" == "ind" {
			local corr "exch 0.0"
		}
	
		local nc : word count `corr'
		forvalues i=1/`nc' {
			local c`i' : word `i' of `corr'
		}

		tempname R
		if "`c1'" == "ar" | "`c1'" == "ar1" {
			MakeBinCorrMatAR   `R' `t' `c2'
		}
		if "`c1'" == "exch" {
			MakeBinCorrMatExch `R' `t' `c2'
		}
		if "`c1'" == "unst" {
			MakeBinCorrMatUnst `R' `t' `c2'
			local t = rowsof(`R')
		}
		drop _all
		noi MakeBinary `n' `t' `R' `probit'
	}
end


program define MakeBinCorrMatUnst
	args R t mat
	
	matrix `R' = matrix(`mat')
	local nr = rowsof(`R')
	local nc = rowsof(`R')
	if `nr' != `nc' {
		noi di as err "correlation matrix is not square"
		exit 198
	}
	
	forvalues i=1/`nr' {
		forvalues j=`i'/`nr' {
			if `R'[`i',`j'] < 0.0 | `R'[`i',`j'] > 1.0 | `R'[`i',`j'] != `R'[`j',`i'] {
				noi di as err "Illegal values in correlation matrix"
				exit 198
			}
		}
	}
end

program define MakeBinCorrMatAR
	args R t alpha
	
	if `alpha' < 0 | `alpha' > 0.60 {
		noi di as err "Autoregressive correlation out of range [0.0, 0.6]"
		exit 198
	}
	
	matrix `R' = I(`t')
	forvalues i=1/`t' {
		forvalues j=`i'/`t' {
			matrix `R'[`i',`j'] = (`alpha')^(abs(`j'-`i'))
			matrix `R'[`j',`i'] = `R'[`i',`j']
		}
	}
end

program define MakeBinCorrMatExch
	args R t alpha
	
	if `alpha' < 0 | `alpha' > 0.60 {
		noi di as err "Exchangeable correlation out of range [0.0, 0.6]"
		exit 198
	}

	matrix `R' = (1-`alpha')*I(`t') + J(`t',`t',`alpha')
end

program define MakeBinary
	args n t R probit
	
	quietly {
		local pow     = 2^`t'
		local ntarget = `n'
		local nn      = int(`n'/`pow') 
		
		if `pow'*`nn' < `n' {
			local nn = `nn'+1 
		}

		local ndots 1
		noi di as txt "Creating datasets for " as res `pow' as txt " covariate patterns"
		noi di
		noi di as txt "----+----1----+----2----+----3----+----4----+----5"
		noi di as txt "." _c
		tempname master junk
		set obs `nn'
		MakeBinaryWork 1 `nn' `t' `R' `probit'
		save `master', replace

		forvalues i=2/`pow' {
			drop _all
			set obs `nn'
			local ndots = `ndots'+1
			local char "."
			if mod(`ndots',5)==0 {
				local char "+"
			}
			if mod(`ndots',10)==0 {
				local char "0"
			}
			noi di as txt "`char'" _c
			if mod(`ndots',50)==0 {
				noi di 
			}
			MakeBinaryWork `i' `nn' `t' `R' `probit'
			save `junk', replace
			use `master', clear
			append using `junk'
			save `master', replace
		}
		noi di _n
		
		use `master', clear
		gen id = _n
		reshape i id
		reshape j t
		reshape xij y a b
		reshape long
		
		local last = `ntarget'*`t'
		keep in 1/`last'
		
		order id t y a b
		label var y  "Outcome variable"
		label var a  "Bernoulli(p~0.50) covariate"
		label var b  "Time-dependent (t-1) covariate"
		label var id "Panel identification number"
		label var t  "Time indicator"
	}	
end


program define MakeBinaryWork
	args xxx n t R probit
	quietly {
		local eps     = 0
		local maxiter = 25000
		
		local tm1 = `t'-1
		local xstr ""
		local xin = `xxx'-1
		forvalues j=1/`t' {
			local i = `t'-`j'
			local digit =  (2^`i' <= `xin')
			local xstr "`xstr' `digit'"
			if `digit' {
				local xin = mod(`xin',2^`i')
			}
		}
		
		forvalues i=1/`t' {
			local xval : word `i' of `xstr'
			local tval = `i'-1 
			local eta  = 0.25 - 0.25*`xval' - 0.25*`tval'
			if "`probit'" == "" {
				local p`i'=1/(1+exp(-(`eta')))
			}
			else {
				local p`i' = normprob((`eta'))
			}
			local q`i' = 1-`p`i''
			forvalues j=`i'/`t' {
				local rho`i'`j' = `R'[`i',`j']
			}
		}	

		local m=0
		forvalues i=1/`t' {
			forvalues j=`i'/`t' {
				local alpha`i'`j' = log(1+`rho`i'`j''*sqrt(`q`i''/`p`i''*`q`j''/`p`j'')) 
			}
			local S`m' "`S`m'' `i'"
		}

		local iter = 1
		local flag = 1
		* Step 1
		*  Let l=l+1
		while `flag' & `iter' < `maxiter' {
			local min=1
			forvalues i=1/`t' {
				forvalues j=1/`t' {
					if `i' <= `j' {
						if `alpha`i'`j'' < `min' & `alpha`i'`j'' > `eps' {
							local min = `alpha`i'`j''
							local r   = `i'
							local s   = `j'
						}
					}
				}
			}
			local m=`m'+1
		
			local beta`m' = `min'	// Let beta`m' = smallest element
			if `alpha`r'`r'' <= `eps' | `alpha`s'`s'' <= `eps' {
				forvalues u=`m'/`t' {
					local beta`u' = 0 
				}
				local m = `t'+1
				local beta`m' = 0 
			} 
			else {
				local S`m' "`r'"
				if `r'!=`s' {
					local S`m' "`S`m'' `s'"
				}
				forvalues i=1/`t' {
					if index("`S`m''","`i'") == 0 {
						local add 1
						foreach j in `S`m'' {
							local x0 = min(`i',`j')
							local x1 = max(`i',`j')
							if `alpha`x0'`x1'' <= `eps' {
								local add 0
							}
						}
						if `add' {
							local S`m' "`S`m'' `i'"
						}
					}
				}
				* Step 2	
				local stop = 1
				foreach i in `S`m'' { 
					foreach j in `S`m'' {
						if `i' <= `j' {
							local alpha`i'`j' = `alpha`i'`j'' - `beta`m''
						}
					}
				}
				forvalues i=1/`t' {
					local q`i' = 1-`p`i''
					local min = 1
					forvalues j=`i'/`t' { 
						if `alpha`i'`j'' > `eps' {
							local stop = 0 
						}
					}
				}
			
				if `stop' {
					local flag = 0
					* goto step 3
				}
			}
			local iter=`iter'+1
		}
		
		if `iter' >= `maxiter' {
			noi di as err _n "Correlation and size are not tractable under this data method"
			exit 199
		}

		forvalues i=1/`m' {
			tempvar X`i'
			gen `X`i'' = rpoisson(`beta`i'')
		}
		
		* Step 3
		forvalues i=1/`t' {
			local exp "0"
			forvalues j=1/`m' {
				if index("`S`j''","`i'") > `eps' {
					local exp "`exp'+`X`j''"
				}
			}
			local val : word `i' of `xstr'
			gen byte y`i'=((`exp')==0)
			gen byte a`i'=(`val')
			gen byte b`i'=(`i'-1)
		}
	}
end

